@@ -15,20 +15,26 @@
 Weights and Biases Logger
 -------------------------
 """
+import operator
 import os
 from argparse import Namespace
+from pathlib import Path
 from typing import Any, Dict, Optional, Union
+from weakref import ReferenceType
 
 import torch.nn as nn
 
+from pytorch_lightning.callbacks.model_checkpoint import ModelCheckpoint
 from pytorch_lightning.loggers.base import LightningLoggerBase, rank_zero_experiment
 from pytorch_lightning.utilities import _module_available, rank_zero_only
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
+from pytorch_lightning.utilities.imports import _compare_version
 from pytorch_lightning.utilities.warnings import WarningCache
 
 warning_cache = WarningCache()
 
 _WANDB_AVAILABLE = _module_available("wandb")
+_WANDB_GREATER_EQUAL_0_10_22 = _compare_version("wandb", operator.ge, "0.10.22")
 
 try:
     import wandb
@@ -40,7 +46,7 @@ except ImportError:
 
 class WandbLogger(LightningLoggerBase):
     r"""
-    Log using `Weights and Biases <https://www.wandb.com/>`_.
+    Log using `Weights and Biases <https://docs.wandb.ai/integrations/lightning>`_.
 
     Install it with pip:
 
@@ -56,7 +62,15 @@ class WandbLogger(LightningLoggerBase):
         version: Same as id.
         anonymous: Enables or explicitly disables anonymous logging.
         project: The name of the project to which this run will belong.
-        log_model: Save checkpoints in wandb dir to upload on W&B servers.
+        log_model: Log checkpoints created by :class:`~pytorch_lightning.callbacks.model_checkpoint.ModelCheckpoint`
+            as W&B artifacts.
+
+            * if ``log_model == 'all'``, checkpoints are logged during training.
+            * if ``log_model == True``, checkpoints are logged at the end of training, except when
+              :paramref:`~pytorch_lightning.callbacks.model_checkpoint.ModelCheckpoint.save_top_k` ``== -1``
+              which also logs every checkpoint during training.
+            * if ``log_model == False`` (default), no checkpoint is logged.
+
         prefix: A string to put at the beginning of metric keys.
         experiment: WandB experiment object. Automatically set when creating a run.
         \**kwargs: Arguments passed to :func:`wandb.init` like `entity`, `group`, `tags`, etc.
@@ -71,20 +85,21 @@ class WandbLogger(LightningLoggerBase):
 
         from pytorch_lightning.loggers import WandbLogger
         from pytorch_lightning import Trainer
-        wandb_logger = WandbLogger()
+
+        # instrument experiment with W&B
+        wandb_logger = WandbLogger(project='MNIST', log_model='all')
         trainer = Trainer(logger=wandb_logger)
 
-    Note: When logging manually through `wandb.log` or `trainer.logger.experiment.log`,
-    make sure to use `commit=False` so the logging step does not increase.
+        # log gradients and model topology
+        wandb_logger.watch(model)
 
     See Also:
-        - `Tutorial <https://colab.research.google.com/drive/16d1uctGaw2y9KhGBlINNTsWpmlXdJwRW?usp=sharing>`__
-          on how to use W&B with PyTorch Lightning
+        - `Demo in Google Colab <http://wandb.me/lightning>`__ with model logging
         - `W&B Documentation <https://docs.wandb.ai/integrations/lightning>`__
 
     """
 
-    LOGGER_JOIN_CHAR = '-'
+    LOGGER_JOIN_CHAR = "-"
 
     def __init__(
         self,
@@ -97,21 +112,28 @@ class WandbLogger(LightningLoggerBase):
         project: Optional[str] = None,
         log_model: Optional[bool] = False,
         experiment=None,
-        prefix: Optional[str] = '',
+        prefix: Optional[str] = "",
         sync_step: Optional[bool] = None,
-        **kwargs
+        **kwargs,
     ):
         if wandb is None:
             raise ImportError(
-                'You want to use `wandb` logger which is not installed yet,'  # pragma: no-cover
-                ' install it with `pip install wandb`.'
+                "You want to use `wandb` logger which is not installed yet,"  # pragma: no-cover
+                " install it with `pip install wandb`."
             )
 
         if offline and log_model:
             raise MisconfigurationException(
-                f'Providing log_model={log_model} and offline={offline} is an invalid configuration'
-                ' since model checkpoints cannot be uploaded in offline mode.\n'
-                'Hint: Set `offline=False` to log your model.'
+                f"Providing log_model={log_model} and offline={offline} is an invalid configuration"
+                " since model checkpoints cannot be uploaded in offline mode.\n"
+                "Hint: Set `offline=False` to log your model."
+            )
+
+        if log_model and not _WANDB_GREATER_EQUAL_0_10_22:
+            warning_cache.warn(
+                f"Providing log_model={log_model} requires wandb version >= 0.10.22"
+                " for logging associated model metadata.\n"
+                "Hint: Upgrade with `pip install --ugrade wandb`."
             )
 
         if sync_step is not None:
@@ -125,29 +147,31 @@ class WandbLogger(LightningLoggerBase):
         self._log_model = log_model
         self._prefix = prefix
         self._experiment = experiment
+        self._logged_model_time = {}
+        self._checkpoint_callback = None
         # set wandb init arguments
-        anonymous_lut = {True: 'allow', False: None}
+        anonymous_lut = {True: "allow", False: None}
         self._wandb_init = dict(
             name=name,
             project=project,
             id=version or id,
             dir=save_dir,
-            resume='allow',
-            anonymous=anonymous_lut.get(anonymous, anonymous)
+            resume="allow",
+            anonymous=anonymous_lut.get(anonymous, anonymous),
         )
         self._wandb_init.update(**kwargs)
         # extract parameters
-        self._save_dir = self._wandb_init.get('dir')
-        self._name = self._wandb_init.get('name')
-        self._id = self._wandb_init.get('id')
+        self._save_dir = self._wandb_init.get("dir")
+        self._name = self._wandb_init.get("name")
+        self._id = self._wandb_init.get("id")
 
     def __getstate__(self):
         state = self.__dict__.copy()
         # args needed to reload correct experiment
-        state['_id'] = self._experiment.id if self._experiment is not None else None
+        state["_id"] = self._experiment.id if self._experiment is not None else None
 
         # cannot be pickled
-        state['_experiment'] = None
+        state["_experiment"] = None
         return state
 
     @property
@@ -165,21 +189,17 @@ class WandbLogger(LightningLoggerBase):
         """
         if self._experiment is None:
             if self._offline:
-                os.environ['WANDB_MODE'] = 'dryrun'
+                os.environ["WANDB_MODE"] = "dryrun"
             self._experiment = wandb.init(**self._wandb_init) if wandb.run is None else wandb.run
 
-        # save checkpoints in wandb dir to upload on W&B servers
-        if self._save_dir is None:
-            self._save_dir = self._experiment.dir
-
         # define default x-axis (for latest wandb versions)
         if getattr(self._experiment, "define_metric", None):
             self._experiment.define_metric("trainer/global_step")
-            self._experiment.define_metric("*", step_metric='trainer/global_step', step_sync=True)
+            self._experiment.define_metric("*", step_metric="trainer/global_step", step_sync=True)
 
         return self._experiment
 
-    def watch(self, model: nn.Module, log: str = 'gradients', log_freq: int = 100):
+    def watch(self, model: nn.Module, log: str = "gradients", log_freq: int = 100):
         self.experiment.watch(model, log=log, log_freq=log_freq)
 
     @rank_zero_only
@@ -191,11 +211,11 @@ class WandbLogger(LightningLoggerBase):
 
     @rank_zero_only
     def log_metrics(self, metrics: Dict[str, float], step: Optional[int] = None) -> None:
-        assert rank_zero_only.rank == 0, 'experiment tried to log from global_rank != 0'
+        assert rank_zero_only.rank == 0, "experiment tried to log from global_rank != 0"
 
         metrics = self._add_prefix(metrics)
         if step is not None:
-            self.experiment.log({**metrics, 'trainer/global_step': step})
+            self.experiment.log({**metrics, "trainer/global_step": step})
         else:
             self.experiment.log(metrics)
 
@@ -213,8 +233,58 @@ class WandbLogger(LightningLoggerBase):
         # don't create an experiment if we don't have one
         return self._experiment.id if self._experiment else self._id
 
+    def after_save_checkpoint(self, checkpoint_callback: "ReferenceType[ModelCheckpoint]") -> None:
+        # log checkpoints as artifacts
+        if self._log_model == "all" or self._log_model is True and checkpoint_callback.save_top_k == -1:
+            self._scan_and_log_checkpoints(checkpoint_callback)
+        elif self._log_model is True:
+            self._checkpoint_callback = checkpoint_callback
+
     @rank_zero_only
     def finalize(self, status: str) -> None:
-        # upload all checkpoints from saving dir
-        if self._log_model:
-            wandb.save(os.path.join(self.save_dir, "*.ckpt"))
+        # log checkpoints as artifacts
+        if self._checkpoint_callback:
+            self._scan_and_log_checkpoints(self._checkpoint_callback)
+
+    def _scan_and_log_checkpoints(self, checkpoint_callback: "ReferenceType[ModelCheckpoint]") -> None:
+        # get checkpoints to be saved with associated score
+        checkpoints = {
+            checkpoint_callback.last_model_path: checkpoint_callback.current_score,
+            checkpoint_callback.best_model_path: checkpoint_callback.best_model_score,
+            **checkpoint_callback.best_k_models,
+        }
+        checkpoints = sorted((Path(p).stat().st_mtime, p, s) for p, s in checkpoints.items() if Path(p).is_file())
+        checkpoints = [
+            c for c in checkpoints if c[1] not in self._logged_model_time.keys() or self._logged_model_time[c[1]] < c[0]
+        ]
+
+        # log iteratively all new checkpoints
+        for t, p, s in checkpoints:
+            metadata = (
+                {
+                    "score": s,
+                    "original_filename": Path(p).name,
+                    "ModelCheckpoint": {
+                        k: getattr(checkpoint_callback, k)
+                        for k in [
+                            "monitor",
+                            "mode",
+                            "save_last",
+                            "save_top_k",
+                            "save_weights_only",
+                            "_every_n_train_steps",
+                            "_every_n_val_epochs",
+                        ]
+                        # ensure it does not break if `ModelCheckpoint` args change
+                        if hasattr(checkpoint_callback, k)
+                    },
+                }
+                if _WANDB_GREATER_EQUAL_0_10_22
+                else None
+            )
+            artifact = wandb.Artifact(name=f"model-{self.experiment.id}", type="model", metadata=metadata)
+            artifact.add_file(p, name="model.ckpt")
+            aliases = ["latest", "best"] if p == checkpoint_callback.best_model_path else ["latest"]
+            self.experiment.log_artifact(artifact, aliases=aliases)
+            # remember logged models - timestamp needed in case filename didn't change (lastkckpt or custom name)
+            self._logged_model_time[p] = t
