@@ -17,11 +17,10 @@ import torch
 from torchmetrics import Metric
 from torchmetrics.metric import CompositionalMetric as _CompositionalMetric
 
-from pytorch_lightning.metrics.utils import deprecated_metrics
+from pytorch_lightning.metrics.utils import deprecated_metrics, void
 
 
 class CompositionalMetric(_CompositionalMetric):
-
     @deprecated_metrics(target=_CompositionalMetric)
     def __init__(
         self,
@@ -33,3 +32,4 @@ class CompositionalMetric(_CompositionalMetric):
         .. deprecated::
             Use :class:`torchmetrics.metric.CompositionalMetric`. Will be removed in v1.5.0.
         """
+        void(operator, metric_a, metric_b)
