@@ -14,7 +14,7 @@
 import torch
 from torchmetrics.functional import hamming_distance as _hamming_distance
 
-from pytorch_lightning.metrics.utils import deprecated_metrics
+from pytorch_lightning.metrics.utils import deprecated_metrics, void
 
 
 @deprecated_metrics(target=_hamming_distance)
@@ -23,3 +23,4 @@ def hamming_distance(preds: torch.Tensor, target: torch.Tensor, threshold: float
     .. deprecated::
         Use :func:`torchmetrics.functional.hamming_distance`. Will be removed in v1.5.0.
     """
+    return void(preds, target, threshold)
