@@ -15,11 +15,10 @@ from typing import Any, Callable, Optional
 
 from torchmetrics import MeanSquaredError as _MeanSquaredError
 
-from pytorch_lightning.metrics.utils import deprecated_metrics
+from pytorch_lightning.metrics.utils import deprecated_metrics, void
 
 
 class MeanSquaredError(_MeanSquaredError):
-
     @deprecated_metrics(target=_MeanSquaredError)
     def __init__(
         self,
@@ -34,4 +33,4 @@ class MeanSquaredError(_MeanSquaredError):
         .. deprecated::
             Use :class:`~torchmetrics.MeanSquaredError`. Will be removed in v1.5.0.
         """
-        _ = compute_on_step, dist_sync_on_step, process_group, dist_sync_fn
+        void(compute_on_step, dist_sync_on_step, process_group, dist_sync_fn)
