@@ -15,11 +15,10 @@ from typing import Any, Callable, Optional
 
 from torchmetrics import MeanSquaredLogError as _MeanSquaredLogError
 
-from pytorch_lightning.metrics.utils import deprecated_metrics
+from pytorch_lightning.metrics.utils import deprecated_metrics, void
 
 
 class MeanSquaredLogError(_MeanSquaredLogError):
-
     @deprecated_metrics(target=_MeanSquaredLogError)
     def __init__(
         self,
@@ -34,4 +33,4 @@ class MeanSquaredLogError(_MeanSquaredLogError):
         .. deprecated::
             Use :class:`~torchmetrics.MeanSquaredLogError`. Will be removed in v1.5.0.
         """
-        _ = compute_on_step, dist_sync_on_step, process_group, dist_sync_fn
+        void(compute_on_step, dist_sync_on_step, process_group, dist_sync_fn)
