@@ -10,7 +10,7 @@ log = logging.getLogger(__name__)
 if torch.distributed.is_available():
     from torch.distributed import Backend, broadcast, get_backend, get_rank, GroupMember
 
-# The code underneath is taken from PyTorch ``torch/distributed/distributed_c10d.py``
+# The code underneath is taken from PyTorch `torch/distributed/distributed_c10d.py`
 # and enable broadcasting for PyTorch 1.6 and lower.
 
 
@@ -48,7 +48,7 @@ def _broadcast_object_list(object_list, src=0, group=None):
     my_rank = get_rank()
     # Serialize object_list elements to tensors on src rank.
     if my_rank == src:
-        tensor_list, size_list = zip(*[_object_to_tensor(obj) for obj in object_list])
+        tensor_list, size_list = zip(*(_object_to_tensor(obj) for obj in object_list))
         object_sizes_tensor = torch.cat(size_list)
     else:
         object_sizes_tensor = torch.LongTensor(len(object_list))
@@ -60,7 +60,7 @@ def _broadcast_object_list(object_list, src=0, group=None):
         # See note about using torch.cuda.current_device() here in docstring.
         # We cannot simply use my_rank since rank == device is not necessarily
         # true.
-        current_device = torch.device('cuda', torch.cuda.current_device())
+        current_device = torch.device("cuda", torch.cuda.current_device())
         object_sizes_tensor = object_sizes_tensor.to(current_device)
         object_sizes_tensor = object_sizes_tensor.to(current_device)
 
@@ -82,13 +82,20 @@ def _broadcast_object_list(object_list, src=0, group=None):
     offset = 0
     if my_rank != src:
         for i, obj_size in enumerate(object_sizes_tensor):
-            obj_view = object_tensor[offset:offset + obj_size]
+            obj_view = object_tensor[offset : offset + obj_size]
             obj_view = obj_view.type(torch.ByteTensor)  # type: ignore[call-overload]
             offset += obj_size
             object_list[i] = _tensor_to_object(obj_view, obj_size)
 
 
-if _TORCH_GREATER_EQUAL_1_8 and torch.distributed.is_available():
+if not torch.distributed.is_available():
+    # avoid failures on early PyTorch versions for Windows where
+    # not all functions used in `broadcast_object_list` are available.
+    def _broadcast_noop(obj, *_, **__):
+        return obj
+
+    broadcast_object_list = _broadcast_noop
+elif _TORCH_GREATER_EQUAL_1_8:
     from torch.distributed.distributed_c10d import broadcast_object_list
 else:
     broadcast_object_list = _broadcast_object_list
