@@ -15,7 +15,7 @@
 import inspect
 import logging
 import os
-from functools import partial
+from functools import lru_cache, partial
 from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Set, Type, TYPE_CHECKING, Union
 
@@ -24,9 +24,10 @@ from torch import nn, Tensor
 from torch.autograd.profiler import record_function
 
 from pytorch_lightning.profiler.base import BaseProfiler
-from pytorch_lightning.utilities import rank_zero_deprecation, rank_zero_warn
+from pytorch_lightning.utilities import rank_zero_warn
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
 from pytorch_lightning.utilities.imports import _KINETO_AVAILABLE
+from pytorch_lightning.utilities.warnings import WarningCache
 
 if TYPE_CHECKING:
     from torch.autograd.profiler import EventList
@@ -38,6 +39,7 @@ if _KINETO_AVAILABLE:
     from torch.profiler import ProfilerAction, ProfilerActivity, tensorboard_trace_handler
 
 log = logging.getLogger(__name__)
+warning_cache = WarningCache()
 
 _PROFILER = Union[torch.autograd.profiler.profile, torch.cuda.profiler.profile, torch.autograd.profiler.emit_nvtx]
 
@@ -116,6 +118,7 @@ class ScheduleWrapper:
         self._current_action = current_action
 
     def reset(self):
+        # handle properly `fast_dev_run`. PyTorch Profiler will fail otherwise.
         self._num_optimizer_step_and_closure = 0
         self._num_validation_step = 0
         self._num_test_step = 0
@@ -128,9 +131,15 @@ class ScheduleWrapper:
         self._current_action: Optional[str] = None
         self._start_action_name: Optional[str] = None
 
+    @property
+    def is_training(self) -> bool:
+        return self._current_action is not None and (
+            self._current_action.startswith("optimizer_step_and_closure_") or self._current_action == "training_step"
+        )
+
     @property
     def num_step(self) -> int:
-        if self._current_action is not None and self._current_action.startswith("optimizer_step_and_closure_"):
+        if self.is_training:
             return self._num_optimizer_step_and_closure
         if self._current_action == "validation_step":
             return self._num_validation_step
@@ -141,7 +150,7 @@ class ScheduleWrapper:
         return 0
 
     def _step(self) -> None:
-        if self._current_action is not None and self._current_action.startswith("optimizer_step_and_closure_"):
+        if self.is_training:
             self._num_optimizer_step_and_closure += 1
         elif self._current_action == "validation_step":
             if self._start_action_name == "on_fit_start":
@@ -156,7 +165,7 @@ class ScheduleWrapper:
 
     @property
     def has_finished(self) -> bool:
-        if self._current_action is not None and self._current_action.startswith("optimizer_step_and_closure_"):
+        if self.is_training:
             return self._optimizer_step_and_closure_reached_end
         if self._current_action == "validation_step":
             return self._validation_step_reached_end
@@ -172,9 +181,9 @@ class ScheduleWrapper:
             return ProfilerAction.NONE
 
         self._step()
-        action = self._schedule(self.num_step)
+        action = self._schedule(max(self.num_step, 0))
         if action == ProfilerAction.RECORD_AND_SAVE:
-            if self._current_action is not None and self._current_action.startswith("optimizer_step_and_closure_"):
+            if self.is_training:
                 self._optimizer_step_and_closure_reached_end = True
             elif self._current_action == "validation_step":
                 self._validation_step_reached_end = True
@@ -196,7 +205,7 @@ class PyTorchProfiler(BaseProfiler):
         "predict_step",
     }
     RECORD_FUNCTION_PREFIX = "optimizer_step_and_closure_"
-    STEP_FUNCTIONS = {"validation_step", "test_step", "predict_step"}
+    STEP_FUNCTIONS = {"training_step", "validation_step", "test_step", "predict_step"}
     STEP_FUNCTION_PREFIX = "optimizer_step_and_closure_"
     AVAILABLE_SORT_KEYS = {
         "cpu_time",
@@ -277,19 +286,16 @@ class PyTorchProfiler(BaseProfiler):
         """
         super().__init__(dirpath=dirpath, filename=filename, output_filename=output_filename)
 
-        record_functions = self.__deprecation_check(profiled_functions, record_functions)
-
         self._group_by_input_shapes = group_by_input_shapes and profiler_kwargs.get("record_shapes", False)
         self._emit_nvtx = emit_nvtx
         self._export_to_chrome = export_to_chrome
         self._row_limit = row_limit
         self._sort_by_key = sort_by_key or f"{'cuda' if profiler_kwargs.get('use_cuda', False) else 'cpu'}_time_total"
-        self._user_record_functions = record_functions
+        self._user_record_functions = set(record_functions or set())
         self._record_functions_start = self._user_record_functions | self.START_RECORD_FUNCTIONS
         self._record_functions = self._user_record_functions | self.RECORD_FUNCTIONS
         self._record_module_names = record_module_names
         self._profiler_kwargs = profiler_kwargs
-
         self.profiler: Optional[_PROFILER] = None
         self.function_events: Optional["EventList"] = None
         self._lightning_module: Optional["LightningModule"] = None  # set by ProfilerConnector
@@ -320,6 +326,7 @@ class PyTorchProfiler(BaseProfiler):
                 raise MisconfigurationException(
                     f"Schedule should return a `torch.profiler.ProfilerAction`. Found: {action}"
                 )
+        self._default_schedule()
         schedule = schedule if has_schedule else self._default_schedule()
         self._schedule = ScheduleWrapper(schedule) if schedule is not None else schedule
         self._profiler_kwargs["schedule"] = self._schedule
@@ -331,28 +338,13 @@ class PyTorchProfiler(BaseProfiler):
         with_stack = profiler_kwargs.get("with_stack", False) or self._export_to_flame_graph
         self._profiler_kwargs["with_stack"] = with_stack
 
-    def __deprecation_check(
-        self, profiled_functions: Optional[List[str]], record_functions: Optional[Set[str]]
-    ) -> Set[str]:
-        if record_functions is None:
-            record_functions = set()
-
-        if profiled_functions is not None:
-            rank_zero_deprecation(
-                "`PyTorchProfiler.profiled_functions` has been renamed to"
-                " `record_functions` in v1.3 and will be removed in v1.5"
-            )
-            if not record_functions:
-                record_functions |= set(profiled_functions)
-            else:
-                raise MisconfigurationException(
-                    "You set `PytorchProfiler.profiled_functions` and `PyTorchProfiler.record_functions`."
-                    "  Please use only the later."
-                )
-
-        return record_functions
+    def _should_override_schedule(self) -> bool:
+        return (self._lightning_module is not None and self._lightning_module.trainer.limit_train_batches < 5) and (
+            self._schedule is not None and self._schedule._schedule == self._default_schedule()
+        )
 
     @staticmethod
+    @lru_cache(1)
     def _default_schedule() -> Optional[callable]:
         if _KINETO_AVAILABLE:
             # Those schedule defaults allow the profiling overhead to be negligible over training time.
@@ -393,11 +385,18 @@ class PyTorchProfiler(BaseProfiler):
             if self._register is not None:
                 self._register.__enter__()
 
+        if self._lightning_module is not None:
+            # when the model is used in automatic optimization,
+            # we use `optimizer_step_and_closure` to step the model.
+            if self._lightning_module.automatic_optimization and "training_step" in self.STEP_FUNCTIONS:
+                self.STEP_FUNCTIONS.remove("training_step")
+
         if (
             self.profiler is not None
             and (action_name in self._record_functions or action_name.startswith(self.RECORD_FUNCTION_PREFIX))
             and action_name not in self._recording_map
         ):
+
             recording = record_function(action_name)
             recording.__enter__()
             self._recording_map[action_name] = recording
@@ -413,6 +412,17 @@ class PyTorchProfiler(BaseProfiler):
         if self.profiler is not None and (
             action_name in self.STEP_FUNCTIONS or action_name.startswith(self.STEP_FUNCTION_PREFIX)
         ):
+
+            # the default schedule requires a minimum of 5 steps to properly work: `wait=1, warmup=1, active=3`.
+            # otherwise, this will raise a `segmentation fault`.
+            if self._should_override_schedule():
+                warning_cache.warn(
+                    "The PyTorch Profiler default schedule will be overridden as there is not enough "
+                    "steps to properly record traces."
+                )
+                self._schedule = None
+                self.profiler.schedule = torch.profiler.profiler._default_schedule_fn
+
             if self._schedule is not None:
                 self._schedule.pre_step(action_name)
 
