@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Optional, Union
+from typing import Callable, Optional, Union
 
 import pytorch_lightning as pl
 from pytorch_lightning.trainer.supporters import prefetch_iterator
@@ -117,19 +117,23 @@ class DataConnector:
         # functions to overwrite with these implementations
         if train_dataloaders is not None:
             self.trainer.train_dataloader = None
-            model.train_dataloader = _PatchDataLoader(train_dataloaders)
+            train_dataloader = _PatchDataLoader(train_dataloaders, "train")
+            train_dataloader.patch(model)
 
         if val_dataloaders is not None:
             self.trainer.val_dataloaders = None
-            model.val_dataloader = _PatchDataLoader(val_dataloaders)
+            val_dataloader = _PatchDataLoader(val_dataloaders, "val")
+            val_dataloader.patch(model)
 
         if test_dataloaders is not None:
             self.trainer.test_dataloaders = None
-            model.test_dataloader = _PatchDataLoader(test_dataloaders)
+            test_dataloader = _PatchDataLoader(test_dataloaders, "test")
+            test_dataloader.patch(model)
 
         if predict_dataloaders is not None:
             self.trainer.predict_dataloaders = None
-            model.predict_dataloader = _PatchDataLoader(predict_dataloaders)
+            predict_dataloader = _PatchDataLoader(predict_dataloaders, "predict")
+            predict_dataloader.patch(model)
 
     def attach_datamodule(
         self, model: "pl.LightningModule", datamodule: Optional["pl.LightningDataModule"] = None
@@ -157,6 +161,13 @@ class DataConnector:
         if hasattr(datamodule, "data_pipeline"):
             model.data_pipeline = datamodule.data_pipeline
 
+    @staticmethod
+    def detach_data(model: "pl.LightningModule") -> None:
+        for stage in ("train", "val", "test", "predict"):
+            loader = getattr(model, f"{stage}_dataloader", None)
+            if isinstance(loader, _PatchDataLoader):
+                loader.unpatch(model)
+
 
 class _PatchDataLoader:
     r"""
@@ -167,13 +178,23 @@ class _PatchDataLoader:
         dataloader: Dataloader object to return when called.
     """
 
-    def __init__(self, dataloader: Union[TRAIN_DATALOADERS, EVAL_DATALOADERS]) -> None:
+    def __init__(self, dataloader: Union[TRAIN_DATALOADERS, EVAL_DATALOADERS], stage: str) -> None:
         self.dataloader = dataloader
 
         # cannot pickle __code__ so cannot verify if PatchDataloader
         # exists which shows dataloader methods have been overwritten.
         # so, we hack it by using the string representation
         self.patch_loader_code = str(self.__call__.__code__)
+        self.old_loader: Optional[Callable] = None
+        self.stage = stage
 
     def __call__(self) -> Union[TRAIN_DATALOADERS, EVAL_DATALOADERS]:
         return self.dataloader
+
+    def patch(self, model: "pl.LightningModule") -> None:
+        self._old_loader = getattr(model, self.stage + "_dataloader")
+        setattr(model, self.stage + "_dataloader", self)
+
+    def unpatch(self, model: "pl.LightningModule") -> None:
+        setattr(model, self.stage + "_dataloader", self._old_loader)
+        self._old_loader = None
