@@ -22,8 +22,8 @@ from pytorch_lightning.utilities.imports import _XLA_AVAILABLE
 if _XLA_AVAILABLE:
     import torch_xla.core.xla_model as xm
 
-#: define waiting time got checking TPU available in sec
-TPU_CHECK_TIMEOUT = 25
+# define TPU availability timeout in seconds
+TPU_CHECK_TIMEOUT = 60
 
 
 def inner_f(queue, func, *args, **kwargs):  # pragma: no cover
@@ -36,7 +36,6 @@ def inner_f(queue, func, *args, **kwargs):  # pragma: no cover
 
 
 def pl_multi_process(func):
-
     @functools.wraps(func)
     def wrapper(*args, **kwargs):
         queue = Queue()
@@ -66,7 +65,13 @@ class XLADeviceUtils:
         Return:
             A boolean value indicating if TPU devices are available
         """
-
+        # For the TPU Pod training process, for example, if we have
+        # TPU v3-32 with 4 VMs, the world size would be 4 and as
+        # we would have to use `torch_xla.distributed.xla_dist` for
+        # multiple VMs and TPU_CONFIG won't be available, running
+        # `xm.get_xla_supported_devices("TPU")` won't be possible.
+        if xm.xrt_world_size() > 1:
+            return True
         return len(xm.get_xla_supported_devices("TPU")) > 0
 
     @staticmethod
@@ -87,7 +92,7 @@ class XLADeviceUtils:
         Return:
             A boolean value indicating if a TPU device exists on the system
         """
-        if os.getenv("PL_TPU_AVAILABLE", '0') == "1":
+        if os.getenv("PL_TPU_AVAILABLE", "0") == "1":
             XLADeviceUtils._TPU_AVAILABLE = True
 
         if XLADeviceUtils.xla_available() and not XLADeviceUtils._TPU_AVAILABLE:
@@ -95,6 +100,5 @@ class XLADeviceUtils:
             XLADeviceUtils._TPU_AVAILABLE = XLADeviceUtils._is_device_tpu()
 
             if XLADeviceUtils._TPU_AVAILABLE:
-                os.environ["PL_TPU_AVAILABLE"] = '1'
-
+                os.environ["PL_TPU_AVAILABLE"] = "1"
         return XLADeviceUtils._TPU_AVAILABLE
